"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseResponse = exports.actionToKey = exports.makeRequest = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
async function makeRequest(url, token, actions) {
    const reqBody = JSON.stringify({ actions });
    return node_fetch_1.default(`${url}/api/bpg`, {
        headers: {
            accept: '*/*',
            'accept-language': 'en-US,en;q=0.9',
            'cache-control': 'no-cache',
            'content-type': 'application/json',
            pragma: 'no-cache',
            'x-coroner-location': url,
            'x-coroner-token': token,
        },
        body: reqBody,
        method: 'POST',
    });
}
exports.makeRequest = makeRequest;
function actionToKey(action) {
    return `${action.type}__${action.action}`;
}
exports.actionToKey = actionToKey;
function parseResult(responseResult) {
    const result = {};
    try {
        if (responseResult.text === 'success') {
            result.data = responseResult.result;
        }
        else {
            result.data = [];
            result.errors = [responseResult.text];
        }
    }
    catch (err) {
        result.data = [];
        result.errors = ['unable to parse BPG response'];
    }
    return result;
}
function parseResponse(response) {
    const results = {};
    response.actions.forEach(function (action, index) {
        const key = actionToKey(action);
        const result = parseResult(response.results[index]);
        if (results[key]) {
            const existingData = results[key].data;
            const existingErrors = results[key].errors;
            results[key].data = [...existingData, ...result.data];
            results[key].errors = [...existingErrors, ...result.errors];
        }
        else {
            results[key] = parseResult(response.results[index]);
        }
    });
    return results;
}
exports.parseResponse = parseResponse;
//# sourceMappingURL=backtraceParserGenerator.js.map