"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useTransaction = exports.getDb = exports.knexfile = void 0;
const knexfile_1 = __importDefault(require("../../knexfile"));
exports.knexfile = knexfile_1.default;
const seed_1 = __importDefault(require("./seed"));
const knex_1 = __importDefault(require("knex"));
const child_process_1 = require("child_process");
const path_1 = __importDefault(require("path"));
let _db = null;
async function getDb() {
    if (_db !== null)
        return _db;
    const db = knex_1.default(knexfile_1.default);
    await db.schema.raw('PRAGMA foreign_keys = ON;');
    try {
        await db.migrate.latest();
    }
    catch (e) {
        const filename = knexfile_1.default.connection.filename;
        console.log(e);
        console.error(`Unable to write to database file. Please make sure this process has write permission to this file and start the service again: ${filename}`);
        process.exit(1);
    }
    await seed_1.default(db);
    if (process.env.NODE_ENV === 'development') {
        const { stdout } = child_process_1.spawnSync(path_1.default.resolve('scripts', 'sqleton.sh'));
        if (stdout.toString() !== '') {
            console.log(stdout.toString());
        }
    }
    _db = db;
    return db;
}
exports.getDb = getDb;
async function useTransaction(db, mainFn) {
    const trx = await db.transaction();
    try {
        await mainFn(trx);
        trx.commit();
        return;
    }
    catch (e) {
        trx.rollback();
        throw e;
    }
}
exports.useTransaction = useTransaction;
//# sourceMappingURL=db.js.map