"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDashboardQueryItemsWithConfigs = exports.getQueryConfig = exports.getQueryConfigs = exports.createOrUpdateQueryConfig = void 0;
async function createOrUpdateQueryConfig(db, queryItemId, key, val, data) {
    const existingQueryConfig = await db('query_item_configs').first('id').where({
        query_item_id: queryItemId,
        key: key,
        universe_name: data.universe_name,
        project_name: data.project_name,
    });
    if (existingQueryConfig) {
        await db('query_item_configs')
            .where({ id: existingQueryConfig.id })
            .update({ key, val: JSON.stringify(val) });
    }
    else {
        await db('query_item_configs').insert(Object.assign({ query_item_id: queryItemId, key, val: JSON.stringify(val) }, data));
    }
}
exports.createOrUpdateQueryConfig = createOrUpdateQueryConfig;
async function getQueryConfigs(db, select, where) {
    return (await db('query_item_configs').select(select).where(where)).map((qc) => {
        qc.val = JSON.parse(qc.val);
        return qc;
    });
}
exports.getQueryConfigs = getQueryConfigs;
async function getQueryConfig(db, select, where) {
    const qc = await db('query_item_configs').first(select).where(where);
    qc.val = JSON.parse(qc.val);
    return qc;
}
exports.getQueryConfig = getQueryConfig;
async function getDashboardQueryItemsWithConfigs(db, dashboard_id, universe, project) {
    const queries = {};
    const allQueries = await db('query_items')
        .leftJoin('query_item_configs as c', 'query_items.id', 'c.query_item_id')
        .select([
        'query_items.*',
        'c.id as config_id',
        'c.key as config_key',
        'c.val as config_val',
        'c.universe_name as config_universe_name',
        'c.project_name as config_project_name',
    ])
        .where({
        dashboard_id,
    })
        .andWhere((a) => {
        a.where({
            config_universe_name: `backtrace_default`,
            config_project_name: `backtrace_default`,
        })
            .orWhere({
            config_universe_name: universe,
            config_project_name: project,
        })
            .orWhere({
            config_universe_name: null,
            config_project_name: null,
        });
    });
    for (const q of allQueries) {
        q.config_val = JSON.parse(q.config_val);
        if (queries[q.id]) {
            if (q.config_id) {
                queries[q.id].query_item_configs.push({
                    id: q.config_id,
                    key: q.config_key,
                    val: q.config_val,
                    universe_name: q.config_universe_name,
                    project_name: q.config_project_name,
                });
            }
        }
        else {
            q.query_item_configs = [];
            if (q.config_id) {
                q.query_item_configs = [
                    {
                        id: q.config_id,
                        key: q.config_key,
                        val: q.config_val,
                        universe_name: q.config_universe_name,
                        project_name: q.config_project_name,
                    },
                ];
            }
            delete q.config_id;
            delete q.config_universe_name;
            delete q.config_project_name;
            delete q.config_key;
            delete q.config_val;
            queries[q.id] = q;
        }
    }
    return queries;
}
exports.getDashboardQueryItemsWithConfigs = getDashboardQueryItemsWithConfigs;
//# sourceMappingURL=queryItemConfigs.js.map