"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const db_1 = require("./db");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const queryItemConfigs_1 = require("./queryItemConfigs");
const SEEDFILE_DIR = path_1.default.resolve(__dirname, '..', '..', '..', 'conf', 'seed_data');
async function seed(db) {
    await db_1.useTransaction(db, async (db) => {
        const seedfiles = fs_1.default.readdirSync(SEEDFILE_DIR);
        for (const file of seedfiles) {
            const filepath = path_1.default.resolve(SEEDFILE_DIR, file);
            let data;
            try {
                data = JSON.parse(fs_1.default.readFileSync(filepath).toString());
            }
            catch (e) {
                console.log(`Unable to parse file ${filepath}`);
                continue;
            }
            if (!data.dashboard_seed_id)
                throw `dashboard_seed_id missing for dashboard ${data.id}`;
            await db('dashboards')
                .insert({
                universe_name: 'backtrace_default',
                project_name: 'backtrace_default',
                backtrace_default: true,
                name: data.name,
                id: data.id,
                grid: JSON.stringify(data.gridLayout),
                dashboard_seed_id: data.dashboard_seed_id,
            })
                .onConflict('dashboard_seed_id')
                .merge();
            const { id: dashboard_id } = await db('dashboards')
                .first('id')
                .where({ dashboard_seed_id: data.dashboard_seed_id });
            for (const q of data.queries) {
                if (!q.query_item_seed_id)
                    throw `query_item_seed_id missing for ${q}`;
                q.dashboard_id = dashboard_id;
                const query_layouts = q.layouts;
                const overrides = q.overrides;
                delete q.layouts;
                delete q.overrides;
                await db('query_items')
                    .insert(Object.assign(Object.assign({}, q), { backtrace_default: true }))
                    .onConflict('query_item_seed_id')
                    .merge();
                const { id: query_item_id } = await db('query_items')
                    .first('id')
                    .where({ query_item_seed_id: q.query_item_seed_id });
                for (const size in query_layouts) {
                    query_layouts[size]['i'] = JSON.stringify(query_item_id);
                }
                await db('query_items')
                    .update({ layouts: JSON.stringify(query_layouts) })
                    .where({ id: query_item_id });
                for (const key in overrides) {
                    await queryItemConfigs_1.createOrUpdateQueryConfig(db, query_item_id, key, overrides[key], {
                        backtrace_default: true,
                        universe_name: 'backtrace_default',
                        project_name: 'backtrace_default',
                    });
                }
            }
        }
        return;
    });
}
exports.default = seed;
//# sourceMappingURL=seed.js.map