"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.uniqueAggPointQueryFetch = exports.getGroupedUniqueEvents = exports.getUniqueEvents = exports.composePointQuery = exports.composePointQueryFilter = exports.getUniqueAggregationName = exports.apertureToDPRange = exports.minStoredGranForWidth = exports.STORED_GRANULARITIES = exports.DATAPOINT_GRANULARITIES = void 0;
const coronerdUtils_1 = require("./coronerdUtils");
const query_1 = require("../model/query");
const queryTemplates_1 = require("./queryTemplates");
const crdbResponse_1 = require("./crdbResponse");
const filter_1 = require("../model/query/filter");
const attributeAggregations_1 = require("../model/query/attributeAggregations");
const GRANULARITY_NOT_FOUND = new Error('Granularity not found');
const UNABLE_TO_MAP_APERTURE_TO_DP = new Error('Unable to map time aperture to datapoint range');
const NO_UNIQUE_AGG_OBJECT = new Error('No unique_aggregation object found');
exports.DATAPOINT_GRANULARITIES = [
    1800,
    3600,
    10800,
    21600,
    43200,
    86400,
    7 * 86400,
    31 * 86400,
    92 * 86400,
    366 * 86400,
];
exports.STORED_GRANULARITIES = [
    [1800, 7 * 86400],
    [10800, 31 * 86400],
    [21600, 92 * 86400],
    [86400, 366 * 86400],
    [604800, 5 * 366 * 86400],
];
function minStoredGranForWidth(width) {
    const storedGran = exports.STORED_GRANULARITIES.find(function ([_gran, maxAge]) {
        return width <= maxAge;
    });
    if (!storedGran) {
        throw GRANULARITY_NOT_FOUND;
    }
    return storedGran[0];
}
exports.minStoredGranForWidth = minStoredGranForWidth;
function apertureToDPRange(timeAperture) {
    const [apertureStart, apertureEnd] = timeAperture;
    const duration = apertureEnd - apertureStart;
    let durationGranID = exports.DATAPOINT_GRANULARITIES.findIndex(function (gran) {
        return gran >= duration;
    });
    for (const [alignGran, maxAge] of exports.STORED_GRANULARITIES) {
        if (duration > maxAge) {
            continue;
        }
        const datapointGran = exports.DATAPOINT_GRANULARITIES[durationGranID];
        const datapointAlignGran = minStoredGranForWidth(datapointGran);
        const endBucket = Math.ceil(apertureEnd / datapointAlignGran);
        const end = endBucket * datapointAlignGran;
        let start = end - datapointGran;
        if (duration < 86400) {
            if (start > apertureStart) {
                durationGranID++;
                continue;
            }
        }
        else {
            start -= alignGran;
        }
        return [start, end];
    }
    throw UNABLE_TO_MAP_APERTURE_TO_DP;
}
exports.apertureToDPRange = apertureToDPRange;
function getDurationFromFilter(filters) {
    var _a, _b, _c;
    return ((_c = (_b = (_a = filters === null || filters === void 0 ? void 0 : filters[0]) === null || _a === void 0 ? void 0 : _a._duration.find(function ([operater, _value]) {
        return operater === filter_1.Operater.EQUAL;
    })) === null || _b === void 0 ? void 0 : _b[1]) !== null && _c !== void 0 ? _c : 1800);
}
function getTable(duration) {
    return duration <= 3600 ? 'unique_aggregations' : 'unique_aggregations_coarse';
}
async function getUniqueAggregationName(coronerUrl, token, projectName, universe, config, uniqueGroup) {
    var _a, _b, _c, _d, _e, _f;
    if (uniqueGroup && typeof uniqueGroup === 'string') {
        return uniqueGroup;
    }
    const uniqueAggregations = await coronerdUtils_1.fetchBPGGet(coronerUrl, token, coronerdUtils_1.GET_UNIQUE_AGGREGATIONS);
    const currentProjectObject = (_c = (_b = (_a = config === null || config === void 0 ? void 0 : config.universes) === null || _a === void 0 ? void 0 : _a[universe]) === null || _b === void 0 ? void 0 : _b.projects_ext) === null || _c === void 0 ? void 0 : _c.find(function (project) {
        return project.name === projectName;
    });
    const uniqueAggForProject = uniqueAggregations.find(function (uniqueAgg) {
        return uniqueAgg.project === currentProjectObject.pid;
    });
    if (!uniqueAggForProject) {
        throw NO_UNIQUE_AGG_OBJECT;
    }
    const uniqueAggconfig = JSON.parse(uniqueAggForProject.configuration);
    const uniqueName = (_f = (_e = (_d = uniqueAggconfig === null || uniqueAggconfig === void 0 ? void 0 : uniqueAggconfig.unique) === null || _d === void 0 ? void 0 : _d[0]) === null || _e === void 0 ? void 0 : _e.unique_attributes) === null || _f === void 0 ? void 0 : _f[0];
    return uniqueName;
}
exports.getUniqueAggregationName = getUniqueAggregationName;
async function composePointQueryFilter(coronerUrl, token, projectName, filters, uniqueName) {
    const filtersObject = {};
    const timeAperture = query_1.getTimeAperture(filters);
    const datapointRange = apertureToDPRange(timeAperture);
    const [start, end] = datapointRange;
    const duration = end - start;
    filtersObject._duration = [[filter_1.Operater.EQUAL, duration]];
    filtersObject._end_timestamp = [[filter_1.Operater.EQUAL, end]];
    filtersObject._name = [[filter_1.Operater.EQUAL, uniqueName]];
    const table = getTable(duration);
    const describeResp = await coronerdUtils_1.fetchDescribe(coronerUrl, token, projectName, table);
    const attributes = describeResp.map(function (attribute) {
        return attribute.name;
    });
    Object.keys(filters[0]).forEach(function (filterAttribute) {
        if (attributes.includes(filterAttribute)) {
            filtersObject[filterAttribute] = filters[0][filterAttribute];
        }
    });
    return filtersObject;
}
exports.composePointQueryFilter = composePointQueryFilter;
async function composePointQuery(coronerUrl, token, projectName, filters, uniqueGroupName, group) {
    const filter = await composePointQueryFilter(coronerUrl, token, projectName, filters, uniqueGroupName);
    const duration = getDurationFromFilter([filter]);
    const table = getTable(duration);
    let query;
    if (group) {
        query = queryTemplates_1.getGroupedUniqueAggPointQuery(group);
    }
    else {
        query = queryTemplates_1.getUniqueAggPointQuery();
    }
    query.filter = [filter];
    query.table = table;
    return query;
}
exports.composePointQuery = composePointQuery;
async function getNumeratorCount(coronerUrl, token, projectName, filters, uniqueGroupName) {
    var _a;
    const query = queryTemplates_1.getUniqueAttributeQuery(uniqueGroupName);
    query.filter = filters;
    const crdbResp = await coronerdUtils_1.fetchCrdb(token, coronerUrl, projectName, JSON.stringify(query));
    coronerdUtils_1.parseCrdbResponseForError(crdbResp);
    const group = (_a = crdbResponse_1.parseAggregationRows(crdbResp.response)) === null || _a === void 0 ? void 0 : _a[0];
    const uniqueEvents = crdbResponse_1.unpackAggregation(group, uniqueGroupName, attributeAggregations_1.AggregationOp.UNIQUE, 0);
    return [uniqueGroupName, uniqueEvents];
}
function linearizeCount(filters, count) {
    const timeAperture = query_1.getTimeAperture(filters);
    const datapointRange = apertureToDPRange(timeAperture);
    const [apertureStart, apertureEnd] = timeAperture;
    const [dpStart, dpEnd] = datapointRange;
    const ratio = (apertureEnd - apertureStart) / (dpEnd - dpStart);
    return Math.floor(ratio * count);
}
async function getUniqueEvents(coronerUrl, token, projectName, filters, uniqueGroupName) {
    var _a;
    const query = await composePointQuery(coronerUrl, token, projectName, filters, uniqueGroupName);
    const crdbSumResp = await coronerdUtils_1.fetchCrdb(token, coronerUrl, projectName, JSON.stringify(query));
    coronerdUtils_1.parseCrdbResponseForError(crdbSumResp.response);
    const groupRow = (_a = crdbResponse_1.parseAggregationRows(crdbSumResp.response)) === null || _a === void 0 ? void 0 : _a[0];
    const uniqueEventCount = crdbResponse_1.unpackAggregation(groupRow, '_count', attributeAggregations_1.AggregationOp.SUM, 0);
    const linearizedCount = linearizeCount(filters, uniqueEventCount);
    return [uniqueGroupName, linearizedCount];
}
exports.getUniqueEvents = getUniqueEvents;
async function getGroupedUniqueEvents(coronerUrl, token, projectName, filters, uniqueGroupName, group) {
    const query = await composePointQuery(coronerUrl, token, projectName, filters, uniqueGroupName, group);
    const crdbSumResp = await coronerdUtils_1.fetchCrdb(token, coronerUrl, projectName, JSON.stringify(query));
    coronerdUtils_1.parseCrdbResponseForError(crdbSumResp.response);
    const groupRows = crdbResponse_1.parseAggregationRows(crdbSumResp.response);
    return groupRows.map(function (group) {
        const uniqueEventCount = crdbResponse_1.unpackAggregation(group, '_count', attributeAggregations_1.AggregationOp.SUM, 0);
        const linearizedCount = linearizeCount(filters, uniqueEventCount);
        return { group: group.group, unique: uniqueGroupName, count: linearizedCount };
    });
}
exports.getGroupedUniqueEvents = getGroupedUniqueEvents;
async function uniqueAggPointQueryFetch(coronerUrl, token, projectName, universe, filters, config, inverse = true, uniqueGroup) {
    try {
        const uniqueGroupName = await getUniqueAggregationName(coronerUrl, token, projectName, universe, config, uniqueGroup);
        const [numeratorGroupName, numeratorCount] = await getNumeratorCount(coronerUrl, token, projectName, filters, uniqueGroupName);
        const [denominatorGroupName, denominatorCount] = await getUniqueEvents(coronerUrl, token, projectName, filters, uniqueGroupName);
        const response = {
            title: 'Crash free users',
            inverse,
            denominatorGroupName,
            denominatorCount,
            numeratorGroupName,
            numeratorCount,
        };
        return {
            response,
            status: 200,
            statusText: 'success',
        };
    }
    catch (error) {
        const response = {
            title: '',
            inverse,
            denominatorGroupName: '',
            denominatorCount: 0,
            numeratorGroupName: '',
            numeratorCount: 0,
            error: error.message,
        };
        return {
            response,
            status: 200,
            statusText: 'failure',
        };
    }
}
exports.uniqueAggPointQueryFetch = uniqueAggPointQueryFetch;
//# sourceMappingURL=uniqueAggregations.js.map