"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addQuantizeUintVCol = exports.getTimeAperture = exports.addAggregation = exports.removeAggregation = exports.getEmptyQuery = void 0;
const attributeAggregations_1 = require("./attributeAggregations");
const virtualColumn_1 = require("./virtualColumn");
const filter_1 = require("./filter");
const virtualColumn_2 = require("./virtualColumn");
function getEmptyQuery() {
    return {
        filter: undefined,
        fold: undefined,
        order: undefined,
        group: undefined,
        offset: undefined,
        limit: undefined,
        select: undefined,
        table: undefined,
    };
}
exports.getEmptyQuery = getEmptyQuery;
function removeAggregation(query, attribute, aggOp) {
    if (query.fold && query.fold[attribute]) {
        const index = query.fold[attribute].findIndex(function (agg) {
            return attributeAggregations_1.getAggregationOp(agg) === aggOp;
        });
        if (index > -1) {
            query.fold[attribute].splice(index, 1);
        }
    }
}
exports.removeAggregation = removeAggregation;
function addAggregation(query, attribute, aggregation) {
    if (query.fold && query.fold[attribute]) {
        query.fold[attribute].push(aggregation);
    }
    else if (query.fold) {
        query.fold[attribute] = [aggregation];
    }
    else {
        query.fold = {
            [attribute]: [aggregation],
        };
    }
}
exports.addAggregation = addAggregation;
function getTimeAperture(filters) {
    var _a, _b, _c;
    const timestampRange = (_a = filters === null || filters === void 0 ? void 0 : filters[0]) === null || _a === void 0 ? void 0 : _a.timestamp;
    let rangeStart = (_b = timestampRange === null || timestampRange === void 0 ? void 0 : timestampRange.find((a) => a[0] === filter_1.Operater.AT_LEAST)) === null || _b === void 0 ? void 0 : _b[1];
    let rangeEnd = (_c = timestampRange === null || timestampRange === void 0 ? void 0 : timestampRange.find((a) => a[0] === filter_1.Operater.AT_MOST)) === null || _c === void 0 ? void 0 : _c[1];
    if (!rangeStart || !rangeEnd) {
        throw new Error('Time aperture not found');
    }
    if (typeof rangeStart === 'string' && typeof rangeEnd === 'string') {
        rangeStart = Number.parseInt(rangeStart);
        rangeEnd = Number.parseInt(rangeEnd);
    }
    return [rangeStart, rangeEnd];
}
exports.getTimeAperture = getTimeAperture;
function getGranDuration(duration) {
    let i = 0;
    while (i < virtualColumn_1.preferredQuantizeSizeForDurations.length) {
        const [durationLimit, granularity] = virtualColumn_1.preferredQuantizeSizeForDurations[i];
        if (durationLimit < duration) {
            i++;
            continue;
        }
        else {
            return granularity;
        }
    }
    throw Error('granularity not found');
}
function addQuantizeUintVCol(query, name, timezoneOffset = 0) {
    if (!query.filter) {
        throw Error('No filter found');
    }
    const [apertureStart, apertureEnd] = getTimeAperture(query.filter);
    const duration = Number(apertureEnd) - Number(apertureStart);
    const granDuration = getGranDuration(duration);
    query.virtual_columns = [
        {
            name,
            type: virtualColumn_2.ColumnType.QUANTIZE_UINT,
            quantize_uint: {
                size: granDuration,
                offset: timezoneOffset,
                backing_column: 'timestamp',
            },
        },
    ];
    const rangeStart = Math.floor(apertureStart / granDuration) * granDuration;
    const rangeEnd = Math.ceil(apertureEnd / granDuration) * granDuration;
    return { granDuration, range: [rangeStart, rangeEnd] };
}
exports.addQuantizeUintVCol = addQuantizeUintVCol;
//# sourceMappingURL=index.js.map