'use strict';
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const knexfile_1 = __importDefault(require("../knexfile"));
const knex_1 = __importDefault(require("knex"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const coronerdUtils = require("../src/lib/coronerdUtils");
const queryUtils = require("../src/lib/queryUtils");
const summedEvents = require("../src/lib/summedEvents");
const { composeMetricSumQuery, composeMetricsTableBaseFilter, composeMetricDistributionQuery, } = summedEvents;
const { parseQuery, composeBaseQueryBody, addTimestampRangeToBin } = queryUtils;
let db = null;
beforeAll(async () => {
    try {
        fs_1.default.unlinkSync(path_1.default.resolve('database', 'overview.test.db'));
        db = knex_1.default(knexfile_1.default);
        const log = console.log;
        console.log = () => {
            return undefined;
        };
        await db.migrate.latest();
        console.log = log;
    }
    catch (e) {
        null;
    }
});
afterAll(async () => {
    try {
        fs_1.default.unlinkSync(path_1.default.resolve('database', 'overview.test.db'));
    }
    catch (e) {
        null;
    }
});
describe('queryUtils', () => {
    const coronerUrl = 'test_url';
    const token = 'test_token';
    const project = 'test_project';
    const metricGroup = 'test_metric_group';
    const filters = [
        {
            timestamp: [
                ['at-least', 1622731335696],
                ['at-most', 1622760000000],
            ],
            metric1: ['equal', 'foo'],
            metric2: ['not-equal', 'bar'],
            metric3: ['contains', 'baz'],
        },
    ];
    const versionQuery = '{"group":["<%versionAttribute%>"],"fold":{"timestamp":[["bin", 32]]},"order":[{"name":";group","ordering":"descending"}],"offset":0,"limit":20}';
    const order = [{ name: 'test', ordering: 'ascending' }];
    test('composeMetricsTableBaseFilter()', async () => {
        jest.spyOn(coronerdUtils, 'fetchDescribe').mockImplementation(async function () {
            return [{ name: 'metric1' }, { name: 'metric2' }];
        });
        const f = await composeMetricsTableBaseFilter(coronerUrl, token, project, filters);
        expect(f).toMatchObject({ timestamp: filters[0].timestamp });
        expect(f).toMatchObject({
            metric1: filters[0].metric1,
            metric2: filters[0].metric2,
        });
        expect(f).not.toMatchObject({
            metric3: filters[0].metric3,
        });
    });
    test('composeMetricSumQuery()', async () => {
        var _a;
        jest.spyOn(coronerdUtils, 'fetchDescribe').mockImplementation(async function () {
            return [{ name: 'metric1' }, { name: 'metric2' }];
        });
        const q = await composeMetricSumQuery(coronerUrl, token, project, filters, metricGroup);
        expect(q).toMatchObject({
            filter: [
                {
                    _metric_group: [['equal', metricGroup]],
                    timestamp: filters[0].timestamp,
                    metric1: filters[0].metric1,
                    metric2: filters[0].metric2,
                },
            ],
            group: ['*'],
            fold: { _value: [['sum']] },
            table: 'metrics',
        });
        expect((_a = q === null || q === void 0 ? void 0 : q.filter) === null || _a === void 0 ? void 0 : _a[0]).not.toMatchObject({
            metric3: filters[0].metric3,
        });
    });
    test('composeMetricDistributionQuery()', async () => {
        jest.spyOn(coronerdUtils, 'fetchDescribe').mockImplementation(async function () {
            return [{ name: 'metric1' }, { name: 'metric2' }];
        });
        const q = await composeMetricDistributionQuery(coronerUrl, token, project, filters);
        expect(q).toMatchObject({
            filter: composeMetricsTableBaseFilter(coronerUrl, token, project, filters),
            group: ['_metric_group'],
            fold: { _value: [['sum']] },
            table: 'metrics',
        });
    });
    test('composeBaseQueryBody()', async () => {
        var _a, _b;
        const q = composeBaseQueryBody(versionQuery, filters, order, 'test_attr', undefined, []);
        expect(q.group).toMatchObject(['test_attr']);
        expect(q.order).toBe(order);
        expect(q.filter).toBe(filters);
        const q2 = composeBaseQueryBody(versionQuery, filters, order, 'test_attr', ['my_group'], []);
        expect(q2.group).toMatchObject(['my_group']);
        expect((_a = q2 === null || q2 === void 0 ? void 0 : q2.fold) === null || _a === void 0 ? void 0 : _a.timestamp).toMatchObject([['bin', 32, 1622731335696, 1622760000000]]);
        const q3 = composeBaseQueryBody(versionQuery, [], order, 'test_attr', ['my_group'], []);
        expect((_b = q3 === null || q3 === void 0 ? void 0 : q3.fold) === null || _b === void 0 ? void 0 : _b.timestamp).toMatchObject([['bin', 32]]);
    });
    test('parseQuery()', async () => {
        parseQuery(versionQuery, undefined);
        expect(parseQuery(versionQuery, 'test_attr').group).toMatchObject(['test_attr']);
        expect(parseQuery(versionQuery, undefined).group).toMatchObject(['<%versionAttribute%>']);
    });
    test('addTimestampRangeToBin()', async () => {
        var _a, _b;
        const q = parseQuery(versionQuery, undefined);
        addTimestampRangeToBin(q, 'timestamp', []);
        expect((_a = q.fold) === null || _a === void 0 ? void 0 : _a.timestamp).toMatchObject([['bin', 32]]);
        addTimestampRangeToBin(q, 'timestamp', filters);
        expect((_b = q.fold) === null || _b === void 0 ? void 0 : _b.timestamp).toMatchObject([['bin', 32, 1622731335696, 1622760000000]]);
    });
});
//# sourceMappingURL=overview.test.js.map