import * as Knex from 'knex';

export async function up(knex: Knex): Promise<void> {
  return knex.schema.createTable('query_items', function (t) {
    t.increments('id').unsigned().primary().defaultTo(1);
    t.timestamps(false, true);

    t.integer('dashboard_id').notNullable();
    t.foreign('dashboard_id')
      .references('id')
      .inTable('dashboards')
      .onUpdate('CASCADE') // If dashboard PK is changed, update FK as well.
      .onDelete('CASCADE'); // If dashboard is deleted, delete query_item as well.
    t.boolean('backtrace_default').notNullable().defaultTo(false);
    t.index(['backtrace_default']);
    t.integer('query_item_seed_id').unique(); // unique but nullable

    t.string('name').notNullable();
    t.string('variant').notNullable();
    t.text('query');
    t.text('layouts');
  });
}

export async function down(knex: Knex): Promise<void> {
  return knex.schema.dropTable('query_items');
}
